use std::sync::Arc;

use axum::{extract::Path, Extension, Json};
use eyre::Result;
use foxchat::{
    http::ApiError,
    model::{http::channel::CreateMessageParams, Message, user::PartialUser},
    FoxError, ulid_timestamp, id::ChannelType, Id,
};

use crate::{
    app_state::AppState,
    db::{channel::get_channel, guild::get_guild, message::create_message},
    fed::FoxRequestData,
    model::user::User,
};

pub async fn post_messages(
    Extension(state): Extension<Arc<AppState>>,
    Path(channel_id): Path<Id<ChannelType>>,
    request: FoxRequestData,
    Json(params): Json<CreateMessageParams>,
) -> Result<Json<Message>, ApiError> {
    let user_id = request.user_id.ok_or(FoxError::MissingUser)?;
    let user = User::get(&state, &request.instance, &user_id).await?;

    let mut tx = state.pool.begin().await?;
    let channel = get_channel(&mut *tx, &channel_id).await?;
    let _guild = get_guild(&mut *tx, &channel.guild_id, &user.id).await?;

    let message = create_message(&mut *tx, &channel.id, &user.id, params).await?;

    tx.commit().await?;

    // TODO: dispatch message create event

    Ok(Json(Message {
        id: message.id.0.clone(),
        channel_id: channel.id.0,
        author: PartialUser {
            id: user.id.0,
            username: user.username,
            instance: request.instance.domain,
        },
        content: Some(message.content),
        created_at: ulid_timestamp(&message.id.0),
    }))
}
