mod api;
mod hello;
mod ws;

use crate::{app_state::AppState, config::Config, model::instance::Instance};
use axum::{routing::{post, get}, Extension, Router};
use foxchat::s2s::Dispatch;
use sqlx::{Pool, Postgres};
use tokio::sync::broadcast;
use std::sync::Arc;
use tower_http::trace::TraceLayer;

pub fn new(pool: Pool<Postgres>, config: Config, instance: Instance) -> Router {
    let (broadcast, _) = broadcast::channel::<Dispatch>(1024);
    
    let app_state = Arc::new(AppState {
        pool,
        config,
        public_key: instance.public_key,
        private_key: instance.private_key,
        broadcast,
    });

    let app = Router::new()
        .route("/_fox/chat/hello", post(hello::post_hello))
        .route("/_fox/chat/ws", get(ws::handler))
        .merge(api::router())
        .layer(TraceLayer::new_for_http())
        .layer(Extension(app_state));

    return app;
}
