use eyre::Result;
use sqlx::{Pool, Postgres};
use rsa::{RsaPrivateKey, RsaPublicKey, pkcs1::{DecodeRsaPublicKey, DecodeRsaPrivateKey}};

#[derive(Debug, Clone)]
pub struct Instance {
    pub public_key: RsaPublicKey,
    pub private_key: RsaPrivateKey,
}

impl Instance {
    /// Gets the instance's configuration.
    /// This is a singleton row that is always present.
    pub async fn get(pool: &Pool<Postgres>) -> Result<Self> {
        let instance = sqlx::query!("SELECT * FROM instance WHERE id = 1")
            .fetch_one(pool)
            .await?;

        let public_key = RsaPublicKey::from_pkcs1_pem(&instance.public_key)?;
        let private_key = RsaPrivateKey::from_pkcs1_pem(&instance.private_key)?;

        Ok(Self {
            public_key,
            private_key,
        })
    }
}
